@@ -1,9 +1,9 @@
 /*
   gemdropx.c
   
-  Gem Drop for X Window
+  Gem Drop for SDL (X-Window, MSWindows, MacOS, BeOS, etc.) and Qt/Embedded
   
-  v.0.7
+  v.0.9
   
   based directly on GEMDROP.ACT, the original Atari 8-bit version of Gem Drop
   
@@ -16,6 +16,9 @@
   X Window update: July 29, 1998 - April 4, 1999
   SDL mixer fixes: October 31, 1999
   100% SDL version: December 27, 1999
+  SDL fixes: December 29, 1999 - January 26, 2000
+  Sound tweaks: February 11, 2002
+  Qt/Embedded Support: February 11, 2002 - February 12, 2002
 */
 
 
@@ -26,19 +29,22 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <unistd.h>
-#include <SDL/SDL.h>
 
-#ifndef NOJOYSTICK
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <sys/ioctl.h>
-#include <linux/joystick.h>
+#ifndef EMBEDDED
+#include <SDL.h>
+#else
+#include <qtimer.h>
+#include <qpainter.h>
+#include <qpe/qpeapplication.h>
+#include <qwidget.h>
+#include <qobject.h>
 #endif
 
 #ifndef NOSOUND
-#include <mixer.h>
+#include <SDL_mixer.h>
 #endif
 
+#ifndef EMBEDDED
 #include "data/images/nothing.xbm"
 #include "data/images/nothing-mask.xbm"
 #include "data/images/left.xbm"
@@ -47,6 +53,7 @@
 #include "data/images/right-mask.xbm"
 #include "data/images/action.xbm"
 #include "data/images/action-mask.xbm"
+#endif
 
 
 /* Misc. value defines: */
@@ -74,6 +81,14 @@ typedef struct explosion_type {
 } explosion_type;
 
 
+#ifdef EMBEDDED
+typedef struct SDL_Rect { int x, y, w, h; } SDL_Rect;
+typedef struct SDL_Surface { int w, h; QPixmap * image; } SDL_Surface;
+#define Uint8 unsigned char
+#define Uint32 unsigned long
+#endif
+
+
 /* Object specifications: */
 
 #define NUM_NORMAL_BLOCKS 4
@@ -166,6 +181,8 @@ Mix_Music * songs[NUM_MODS];
 #endif
 
 
+#ifndef EMBEDDED
+
 /* Cursors: */
 
 enum {
@@ -184,11 +201,25 @@ Uint8 * cursor_bitmask[NUM_CURSORS] = {
   nothing_mask_bits, left_mask_bits, right_mask_bits, action_mask_bits
 };
 
+#endif
 
-/* Color palette values: */
 
-#define COLOR_DIVIDE 18
-#define MAX_COLORS 256
+#ifdef EMBEDDED
+#define SDLK_ESCAPE 27
+#define SDLK_a 'a'
+#define SDLK_k 'k'
+#define SDLK_l 'l'
+#define SDLK_p 'p'
+#define SDLK_q 'q'
+#define SDLK_z 'z'
+#define SDLK_RETURN 10
+#define SDLK_SPACE 32
+#define SDLK_LAST 999
+#define SDLK_RIGHT '<'
+#define SDLK_LEFT '>'
+#define SDLK_UP '^'
+#define SDLK_DOWN 'v'
+#endif
 
 
 /* Globals: */
@@ -196,41 +227,245 @@ Uint8 * cursor_bitmask[NUM_CURSORS] = {
 explosion_type explosions[NUM_EXPLOSIONS];
 int blocks[HEIGHT][WIDTH], killsx[100], killsy[100];
 int scorevals[15];
-int level, gameover, frozen, carrying, howmany, leveldone, warning,
+int level, gameover, frozen, carrying, howmany, leveldone, _warning,
   whichexplosion, lines, firstround, linesneeded, happy, score, playerx,
-  use_sound, max_level, clicks, no_music, game_playing;
-#ifndef NOJOYSTICK
-int js_fd;
-struct JS_DATA_TYPE js;
-#endif
+  use_sound, use_joystick, max_level, clicks, no_music, game_playing;
+
+
+int windowwidth, windowheight;
 
 
 SDL_Surface * window;
 SDL_Surface * object_pixmaps[NUM_OBJECTS];
+
+#ifndef EMBEDDED
 SDL_Cursor * cursor[NUM_CURSORS];
-int windowwidth, windowheight;
 SDL_Event event;
 SDLKey key;
+SDL_Joystick * js;
+#else
+
+class MainWindow: public QWidget
+{
+//      Q_OBJECT
+        private:
+                QPixmap* buffer;
+                QPainter* bufpainter;
+                QColor palette[2];
+
+                int (*timerhandler)();
+                int (*keypresshandler)(int);
+                int (*keyreleasehandler)(int);
+
+        protected:
+                void timerEvent(QTimerEvent*);
+                void keyPressEvent(QKeyEvent*);
+                void keyReleaseEvent(QKeyEvent*);
+
+        public:
+                MainWindow(const char* name);
+                void setTimerhandler(int (*timerhandler)(void), int ms = 15);
+                void setKeypresshandler(int (*keypresshandler)(int));
+                void setKeyreleasehandler(int (*keyreleasehandler)(int));
+
+                void setBgColor(Q_UINT32 color);
+                void setFgColor(Q_UINT32 color);
+
+                void plot(int x, int y);
+                void flush();
+
+                void blit(QPixmap* image, int x, int y);
+};
+
+MainWindow::MainWindow(const char* name):
+        QWidget(0, name, WType_TopLevel)
+{
+        // Setup the buffer
+        buffer = new QPixmap(240, 320);
+        bufpainter = new QPainter(buffer);
+
+        // Initialize variables
+        setBgColor(0x000000);
+        setFgColor(0xffffff);
+
+        // Initialize handlers
+        timerhandler = NULL;
+        keypresshandler = NULL;
+        keyreleasehandler = NULL;
+
+        // Setup the window
+        resize(230, 280);
+        setCaption(name);
+        bufpainter->eraseRect(0, 0, 240, 320);
+}
+
+void MainWindow::setTimerhandler(int (*th)(void), int ms)
+{
+        timerhandler = th;                  // Install the timer handler
+        startTimer(ms);
+}
+
+void MainWindow::setKeypresshandler(int (*kph)(int))
+{
+        keypresshandler = kph;              // Install the keypress handler
+}
+
+void MainWindow::setKeyreleasehandler(int (*krh)(int))
+{
+        keyreleasehandler = krh;            // Install the keypress handler
+}
+
+void MainWindow::timerEvent(QTimerEvent*)
+{
+        timerhandler();
+}
+
+
+void MainWindow::keyPressEvent(QKeyEvent* e)
+{
+        if(!e->isAutoRepeat() && keypresshandler)
+        {
+                keypresshandler(e->key());
+        }
+}
+
+
+void MainWindow::keyReleaseEvent(QKeyEvent* e)
+{
+        if(!e->isAutoRepeat() && keyreleasehandler)
+        {
+                keyreleasehandler(e->key());
+        }
+}
+
+
+void MainWindow::setBgColor(Q_UINT32 color)
+{
+        palette[0] = QColor((color >> 16 & 0xff), (color >> 8) & 0xff, color & 0xff);
+        bufpainter->setBackgroundColor(palette[0]);
+}
+
+
+void MainWindow::setFgColor(Q_UINT32 color)
+{
+        palette[1] = QColor((color >> 16 & 0xff), (color >> 8) & 0xff, color & 0xff);
+        bufpainter->setPen(palette[1]);
+}
+
+
+void MainWindow::plot(int x, int y)
+{
+        bufpainter->drawPoint(x, y);
+}
+
+
+void MainWindow::flush()
+{
+        QPainter p(this);
+        p.drawPixmap(0, 0, *buffer);
+}
+
+
+void MainWindow::blit(QPixmap* image, int x, int y)
+{
+        bufpainter->drawPixmap(x, y, *image);
+}
+
+
+
+QPEApplication * qapp;
+MainWindow * mw;
+int key;
+#endif
+
+
+#ifdef EMBEDDED
+int SDL_BlitSurface(SDL_Surface *src, SDL_Rect *srcrect,
+		    SDL_Surface *dst, SDL_Rect *dstrect)
+{
+  mw->blit(src->image, dstrect->x, dstrect->y);
+  return(0);
+}
+
+int SDL_UpdateRect(SDL_Surface *screen, int x, int y, int w, int h)
+{
+  printf("$$ SDL_UpdateRect()\n");
+  return(0);
+}
+
+Uint32 SDL_GetTicks(void)
+{
+  struct timeval tv;
+  
+  gettimeofday(&tv, NULL);
+  return (tv.tv_sec * 100000 + tv.tv_usec) / 1000;
+}
+
+void SDL_Delay(Uint32 ms)
+{
+  usleep(ms * 1000);
+}
+
+SDL_Surface * SDL_LoadBMP(const char * file)
+{
+  SDL_Surface * surf;
+  
+  surf = (SDL_Surface *) malloc(sizeof(SDL_Surface));
+  
+  surf->image = new QPixmap();
+  surf->image->load(file);
+  surf->w = (surf->image)->width();
+  surf->h = (surf->image)->height();
+  return(surf);
+}
+
+char * SDL_GetError(void)
+{
+  printf("$$ SDL_GetError()\n");
+  return ("???");
+}
+
+void SDL_Quit(void)
+{
+  exit(1);
+}
+#endif
+
+
+int my_FillRect(SDL_Surface *dst, SDL_Rect *dstrect,
+		Uint8 r, Uint8 g, Uint8 b)
+{
+#ifndef EMBEDDED
+  return (SDL_FillRect(dst, dstrect,
+		       SDL_MapRGB(dst->format, r, g, b)));
+#else
+  printf("$$ my_FillRect()\n");
+  return(0);
+#endif
+}
 
 
 void playsound(int which)
 {
 #ifndef NOSOUND
   int i, found;
+ 
+  if (use_sound)
+  {
+    found = -1;
+ 
+    for (i = 0; i < MIX_CHANNELS; i++)
+      {
+        if (!Mix_Playing(i))
+	  found = i;
+      }
   
-  found = -1;
+    if (found == -1)
+      found = rand() % MIX_CHANNELS;
   
-  for (i = 0; i < MIX_CHANNELS; i++)
-    {
-      if (!Mix_Playing(i))
-	found = i;
-    }
-  
-  if (found == -1)
-    found = rand() % MIX_CHANNELS;
-  
-  Mix_PlayChannel(found, samples[which], 0);
-  /* Mix_Volume(found, 8); */
+    Mix_PlayChannel(found, samples[which], 0);
+    /* Mix_Volume(found, 8); */
+  }
 #endif
 }
 
@@ -254,27 +489,27 @@ void erasewindow(void)
   dest.w = windowwidth;
   dest.h = windowheight;
   
-  SDL_FillRect(window, &dest,
-               SDL_MapRGB(window->format, 0x00, 0x00, 0x00));
+  my_FillRect(window, &dest, 0x00, 0x00, 0x00);
   SDL_UpdateRect(window, dest.x, dest.y, dest.w, dest.h);
 }
 
 
 /* Setup the application: */
 
-void setup(void)
+void setup(int argc, char * argv[])
 {
-  int i, j;
+  int i;
+  char file[128];
+#ifndef EMBEDDED
+  int j;
   Uint8 temp_bitmap[32], temp_bitmask[32];
   Uint8 b;
-  char file[128];
   SDL_Surface * image;
-  SDL_Rect dest;
-#ifndef NOJOYSTICK
-  char axes, buttons;
 #endif
+  SDL_Rect dest;
 
   
+#ifndef EMBEDDED
   if (SDL_Init(SDL_INIT_VIDEO) < 0)
     {
       fprintf(stderr, "Couldn't initialize SDL video.\n%s\n",
@@ -282,19 +517,90 @@ void setup(void)
       exit(1);
     }
   
+  use_joystick = 1;
+  if (SDL_Init(SDL_INIT_JOYSTICK) < 0)
+    {
+      fprintf(stderr, "Warning - Couldn't initialize SDL joystick.\n%s\n",
+	      SDL_GetError());
+      use_joystick = 0;
+    }
+  
+  
+  /* Set key repeat: */
+  
+  SDL_EnableKeyRepeat(100, 30);
+  
+  
+  /* Open joystick: */
+  
+  if (use_joystick)
+    {
+      if (SDL_NumJoysticks() <= 0)
+	{
+	  fprintf(stderr, "Warning - No joysticks.\n");
+	  use_joystick = 0;
+	}
+    }
+  
+  if (use_joystick)
+    {
+      js = SDL_JoystickOpen(0);
+      if (js == NULL)
+	{
+	  fprintf(stderr, "Warning - Can't open joystick 1.\n%s\n",
+		  SDL_GetError());
+	  use_joystick = 0;
+	}
+    }
+  
+  
+  /* Check for proper joystick config: */
+  
+  if (use_joystick)
+    {
+      if (SDL_JoystickNumAxes(js) < 1)
+	{
+	  fprintf(stderr, "Joystick doesn't have enough axes!\n");
+	  use_joystick = 0;
+	}
+    }
+
+  if (use_joystick)
+    {
+      if (SDL_JoystickNumButtons(js) < 2)
+	{
+	  fprintf(stderr, "Joystick doesn't have enough buttons!\n");
+	  use_joystick = 0;
+	}
+    }
+#else
+  qapp = new QPEApplication(argc, argv);
+  mw = new MainWindow("Gem Drop Z");
+#endif
+  
   
   /* Set the size of the window: */
-  
+ 
+#ifndef EMBEDDED
   windowwidth = WIDTH * 48;
   windowheight = (HEIGHT + 2) * 48;
+#else
+  windowwidth = 240;
+  windowheight = 320;
+#endif
   
 
   /* Open window: */
 
+#ifndef EMBEDDED
   window = SDL_SetVideoMode(windowwidth, windowheight, 16, SDL_HWSURFACE);
   SDL_WM_SetCaption("Gem Drop X!", "Gem Drop X");
+#else
+  printf("$$ setup() - open window\n");
+#endif
   
   
+#ifndef EMBEDDED
   /* Make cursors: */
   
   for (i = 0; i < NUM_CURSORS; i++)
@@ -327,6 +633,7 @@ void setup(void)
       cursor[i] = SDL_CreateCursor(temp_bitmap, temp_bitmask,
 				   16, 16, 8, 8);
     }
+#endif
   
   
   /* Load the object graphics: */
@@ -334,17 +641,30 @@ void setup(void)
   for (i = 0; i < NUM_OBJECTS; i++)
     {
       sprintf(file, "%s/images/%s.bmp", DATA_PREFIX, object_filenames[i]);
-      
+  
+#ifndef EMBEDDED      
       image = SDL_LoadBMP(file);
       
       if (image == NULL)
 	{
 	  fprintf(stderr, "%s: can't load: %s\n", file, SDL_GetError());
+	  SDL_Quit();
 	  exit(1);
 	}
-      
+
       object_pixmaps[i] = SDL_DisplayFormat(image);      
       SDL_FreeSurface(image);
+#else
+      object_pixmaps[i] = SDL_LoadBMP(file);
+      
+      if (object_pixmaps[i] == NULL)
+	{
+	  fprintf(stderr, "%s: can't load: %s\n", file, SDL_GetError());
+	  SDL_Quit();
+	  exit(1);
+	}
+#endif
+      
       
       
       if (i == PATIENCE_GFX)
@@ -353,7 +673,7 @@ void setup(void)
 	  dest.y = (windowheight - 16) / 2;
 	  dest.w = 96;
 	  dest.h = 16;
-	  
+	 
 	  SDL_BlitSurface(object_pixmaps[PATIENCE_GFX - 1], NULL,
 			  window, &dest);
 	  SDL_UpdateRect(window, dest.x, dest.y, dest.w, dest.h);
@@ -398,83 +718,66 @@ void setup(void)
   srand(SDL_GetTicks());
   
   
-  /* Setup joystick: */
-
-#ifndef NOJOYSTICK
-  js_fd = open("/dev/js0", 0, O_RDONLY);
-  if (js_fd < 0)
-    {
-      perror("opening joystick");
-    }
-  else
-    {
-      ioctl(js_fd, JSIOCGAXES, &axes);
-      ioctl(js_fd, JSIOCGBUTTONS, &buttons);
-      
-      if (axes < 2)
-	{
-	  fprintf(stderr, "Not enough axes! (%d, need 2!))\n", axes);
-	  exit(1);
-        }
-      
-      if (buttons < 2)
-        {
-          fprintf(stderr, "Not enough buttons! (%d, need 2!))\n", buttons);
-	  exit(1);
-	}
-      
-      fcntl(js_fd, F_SETFL, O_NONBLOCK);
-    }
-#endif
-
-
   /* Setup sound: */
-  
+ 
+  use_sound = 1;
+
 #ifndef NOSOUND
-  if (SDL_Init(SDL_INIT_AUDIO) < 0)
-    {
-      fprintf(stderr, "Couldn't initialize SDL sound.\n%s\n",
-              SDL_GetError());
-      exit(1);
-    }
-  
-  if (Mix_OpenAudio(11025, AUDIO_S8, 2, 512) < 0)
-    {
-      fprintf(stderr,
-              "Warning: Couldn't set 11025 Hz 8-bit stereo audio.\n%s\n",
-              SDL_GetError());
-      exit(1);
-    }
-  
-
-  /* Load sound files: */
-  
-  for (i = 0; i < NUM_SOUNDS; i++)
-    {
-      sprintf(file, "%s/sounds/%s.wav", DATA_PREFIX, sound_filenames[i]);
-      samples[i] = Mix_LoadWAV(file);
-      if (samples[i] == NULL)
-	{
-	  fprintf(stderr, "Couldn't load %s: %s\n", file, SDL_GetError());
-	  exit(1);
-	}
-    }
+  if (use_sound)
+  {
+    if (SDL_Init(SDL_INIT_AUDIO) < 0)
+      {
+        fprintf(stderr, "Warning: Couldn't initialize SDL sound.\n%s\n",
+                SDL_GetError());
+        use_sound = 0;
+      }
+  }
+ 
+  if (use_sound)
+  {
+    if (Mix_OpenAudio(44100, AUDIO_S8, 2, 512) < 0)
+      {
+        fprintf(stderr,
+                "Warning: Couldn't set 11025 Hz 8-bit stereo audio.\n%s\n",
+                SDL_GetError());
+	use_sound = 0;
+      }
+  }
 
 
-  /* Load music files: */
+  if (use_sound)
+  {
+    /* Load sound files: */
   
-  if (no_music == 0)
-    {
-      for (i = 0; i < NUM_MODS; i++)
-	{
-	  sprintf(file, "%s/sounds/%s", DATA_PREFIX, music_filenames[i]);
-	  songs[i] = Mix_LoadMUS(file);
-	  if (songs[i] == NULL)
-	    {
-	      fprintf(stderr, "Couldn't load %s: %s\n", file, SDL_GetError());
-	      exit(1);
-	    }
-	}
+    for (i = 0; i < NUM_SOUNDS; i++)
+      {
+        sprintf(file, "%s/sounds/%s.wav", DATA_PREFIX, sound_filenames[i]);
+        samples[i] = Mix_LoadWAV(file);
+        if (samples[i] == NULL)
+	  {
+	    fprintf(stderr, "Couldn't load %s: %s\n", file, SDL_GetError());
+	    SDL_Quit();
+	    exit(1);
+	  }
+      }
+
+
+    /* Load music files: */
+  
+    if (no_music == 0)
+      {
+        for (i = 0; i < NUM_MODS; i++)
+	  {
+	    sprintf(file, "%s/sounds/%s", DATA_PREFIX, music_filenames[i]);
+	    songs[i] = Mix_LoadMUS(file);
+	    if (songs[i] == NULL)
+	      {
+	        fprintf(stderr, "Couldn't load %s: %s\n", file, SDL_GetError());
+	        SDL_Quit();
+	        exit(1);
+	      }
+	  }
+      }
     }
 #endif
 }
@@ -511,8 +814,7 @@ void shownumber(int num, int x, int y)
 	}
       else
 	{
-	  SDL_FillRect(window, &dest,
-		       SDL_MapRGB(window->format, 0x00, 0x00, 0x00));
+	  my_FillRect(window, &dest, 0x00, 0x00, 0x00);
 	}
       
       SDL_UpdateRect(window, dest.x, dest.y, dest.w, dest.h);
@@ -584,8 +886,7 @@ void drawblockgraphic(int x, int y, int c)
     }
   else
     {
-      SDL_FillRect(window, &dest,
-               SDL_MapRGB(window->format, 0x00, 0x00, 0x00));
+      my_FillRect(window, &dest, 0x00, 0x00, 0x00);
     }
   
   SDL_UpdateRect(window, dest.x, dest.y, dest.w, dest.h);
@@ -601,7 +902,7 @@ void drawstatus()
   else
     drawblockgraphic((WIDTH / 2) * 48, (HEIGHT + 1) * 48, 0);
   
-  if (warning != 0)
+  if (_warning != 0)
     drawblockgraphic((WIDTH / 2) * 48 + 48, (HEIGHT + 1) * 48, WARNING_GFX +
 		     (clicks % 4));
   else
@@ -705,7 +1006,8 @@ void getkey(int handle_expose)
 {
   /* int x, y; */
   
-  
+
+#ifndef EMBEDDED  
   key = SDLK_LAST;
   
   while (SDL_PollEvent(&event))
@@ -763,27 +1065,52 @@ void getkey(int handle_expose)
 	    }
 	}
     }
+#else
+  printf("$$ getkey()\n");
+#endif
 }
 
 
 /* Wait until joystick buttons are released: */
 
+#ifndef EMBEDDED
+
 void eatjoybuttons(void)
 {
-#ifndef NOJOYSTICK
-  do
+  int i, any_down;
+  SDL_Event event;
+  
+  
+  if (use_joystick)
     {
-      read(js_fd, &js, JS_RETURN);
+      do
+	{
+	  any_down = 0;
+
+	  SDL_PollEvent(&event);
+	  
+	  for (i = 0; i < SDL_JoystickNumButtons(js); i++)
+	    {
+	      if (SDL_JoystickGetButton(js, i))
+		any_down = 1;
+	    }
+	  
+	  SDL_Delay(10);
+	}
+      while (any_down);
     }
-  while (js.buttons != 0);
-#endif
 }
 
+#endif
+
 
 /* Show the title... let them change the level: */
 
 int title(void)
 {
+#ifndef EMBEDDED
+  int btn;
+#endif
   int quit, ok, title_width, title_height, height,
     author_width, author_height, author_x, author_y, author,
     author_xm, author_ym, sine_timer, i, credits_width, credits_height;
@@ -801,7 +1128,7 @@ int title(void)
 
   credits_width = object_pixmaps[CREDITS_GFX - 1] -> w;
   credits_height = object_pixmaps[CREDITS_GFX - 1] -> h;
-
+  
   author_x = WIDTH * 48 - author_width;
   author_y = HEIGHT * 48 - author_height - (rand() % 10) - 30;
   
@@ -888,8 +1215,8 @@ int title(void)
 	      src.w = title_width;
 	      src.h = 1;
 	      
-	      dest.x = ((WIDTH * 48 - title_width) / 2 + 
-			 sin(M_PI * ((i + sine_timer) * 5) / 180.0) * 5.0);
+	      dest.x = (int) ((WIDTH * 48 - title_width) / 2 + 
+			sin(M_PI * ((i + sine_timer) * 5) / 180.0) * 5.0);
 	      dest.y = i;
 	      dest.w = title_width;
 	      dest.h = 1;
@@ -912,8 +1239,7 @@ int title(void)
 	  dest.w = author_width;
 	  dest.h = author_height;
 	  
-	  SDL_FillRect(window, &dest,
-               SDL_MapRGB(window->format, 0x00, 0x00, 0x00));
+	  my_FillRect(window, &dest, 0x00, 0x00, 0x00);
 	  
 	  SDL_UpdateRect(window, dest.x, dest.y, dest.w, dest.h);
 	  
@@ -1011,17 +1337,21 @@ int title(void)
 	  else
 	    author = 1495;
 	}
-
-#ifndef NOJOYSTICK      
-      if (js_fd >= 0)
-	{
-	  read(js_fd, &js, JS_RETURN);
       
-	  if (js.buttons != 0)
-	    ok = YES;
+      
+      /* Any joystick buttons? */
+
+#ifndef EMBEDDED      
+      if (use_joystick)
+	{
+	  for (btn = 0; btn < SDL_JoystickNumButtons(js); btn++)
+	    {
+	      if (SDL_JoystickGetButton(js, btn))
+		ok = YES;
+	    }
 	}
 #endif
-
+      
       
       /* Keep framerate exact: */
       
@@ -1031,17 +1361,22 @@ int title(void)
 
       /* Play music: */
 #ifndef NOSOUND
-      if (no_music == 0)
-	{
-	  if (!Mix_PlayingMusic())
-	    Mix_PlayMusic(songs[MOD_TITLE], 0);
-	  /* Mix_VolumeMusic(8); */
-	}
+      if (use_sound)
+      {
+        if (no_music == 0)
+	  {
+	    if (!Mix_PlayingMusic())
+	      Mix_PlayMusic(songs[MOD_TITLE], 0);
+	    /* Mix_VolumeMusic(8); */
+	  }
+      }
 #endif
     }
   while (ok != YES);
   
+#ifndef EMBEDDED
   eatjoybuttons();
+#endif
   
   return(quit);
 }
@@ -1114,7 +1449,7 @@ void initlevel(void)
   carrying = 0;
   howmany = 0;
   frozen = 0;
-  warning = NO;
+  _warning = NO;
   firstround = NO;
   
   
@@ -1149,8 +1484,7 @@ void eraseyou(int x)
   dest.w = 48 * 3;
   dest.h = 48;
   
-  SDL_FillRect(window, &dest,
-               SDL_MapRGB(window->format, 0x00, 0x00, 0x00));
+  my_FillRect(window, &dest, 0x00, 0x00, 0x00);
   
   SDL_UpdateRect(window, dest.x, dest.y, dest.w, dest.h);
 }
@@ -1340,7 +1674,7 @@ int killblock(int x, int y, int killed)
 
 /* Throw your gems: */
 
-void throw(int x)
+void _throw(int x)
 {
   int lasty, killed;
   int y, c, last, nextlast, ytop, ybot, ok, doit;
@@ -1763,8 +2097,7 @@ void level15fx()
       dest.w = 56;
       dest.h = 64;
       
-      SDL_FillRect(window, &dest,
-		   SDL_MapRGB(window->format, 0x00, 0x00, 0x00));
+      my_FillRect(window, &dest, 0x00, 0x00, 0x00);
       SDL_UpdateRect(window, dest.x, dest.y, dest.w, dest.h);
       
       drawmangraphic(x, windowheight / 2 - 16, SAD_GFX);
@@ -1791,7 +2124,11 @@ void level15fx()
 
 void play()
 {
-  int oact, q, x, y, toggle, ojsx, ojsbuttons, jstimer, win;
+  int oact, q, x, y, toggle, win;
+#ifndef EMBEDDED
+  int jsx, ojsx, jsbutton0, jsbutton1, jsbutton23,
+    ojsbutton0, ojsbutton1, ojsbutton23, jstimer;
+#endif
   Uint32 last_time;
   SDL_Rect dest;
 #ifndef NOSOUND
@@ -1815,9 +2152,13 @@ void play()
   drawyou(playerx);
   
   
-  ojsx = 128;
-  ojsbuttons = 0;
+#ifndef EMBEDDED
+  ojsx = 0;
+  ojsbutton0 = 0;
+  ojsbutton1 = 0;
+  ojsbutton23 = 0;
   jstimer = 0;
+#endif
   
   
   /* MAIN GAME LOOP: */
@@ -1831,25 +2172,30 @@ void play()
       
       getkey(1);
 
-#ifndef NOJOYSTICK
-      if (js_fd >= 0)
+#ifndef EMBEDDED
+      if (use_joystick)
 	{
-          read(js_fd, &js, JS_RETURN);
+	  jsx = SDL_JoystickGetAxis(js, 0);
 	  
-          if (js.x < 64 && (ojsx >= 64 || jstimer == STICKWAIT - 1))
+	  if (jsx < -16384 && (ojsx >= -16384 || jstimer == STICKWAIT - 1))
 	    key = SDLK_LEFT;
-          else if (js.x > 192 && (ojsx <= 192 || jstimer == STICKWAIT - 1))
+	  else if (jsx > 16384 && (ojsx <= 16384 || jstimer == STICKWAIT - 1))
 	    key = SDLK_RIGHT;
 	  
-          if ((js.buttons & 0x02) && (ojsbuttons & 0x02) == 0)
+	  jsbutton0 = SDL_JoystickGetButton(js, 0);
+	  jsbutton1 = SDL_JoystickGetButton(js, 1);
+	  jsbutton23 = (SDL_JoystickGetButton(js, 2) +
+			SDL_JoystickGetButton(js, 3));
+	  
+          if (jsbutton1 && ojsbutton1 == 0)
 	    key = SDLK_DOWN;
-          else if ((js.buttons & 0x01) && (ojsbuttons & 0x01) == 0)
+          else if (jsbutton0 && ojsbutton0 == 0)
 	    key = SDLK_UP;
-          else if (((js.buttons & 0x04) && (ojsbuttons & 0x04) == 0) ||
-	           ((js.buttons & 0x08) && (ojsbuttons & 0x08) == 0))
+          else if (jsbutton23 && ojsbutton23 == 0)
 	    key = SDLK_RETURN;
           
-          if (js.x == ojsx)
+          if ((jsx < -16384 && ojsx < -16384) ||
+	      (jsx > 16384 && ojsx > 16384))
 	    {
 	      jstimer++;
 	      if (jstimer >= STICKWAIT)
@@ -1858,8 +2204,10 @@ void play()
           else
 	    jstimer = 0;
 	  
-	  ojsx = js.x;
-	  ojsbuttons = js.buttons;
+	  ojsx = jsx;
+	  ojsbutton0 = jsbutton0;
+	  ojsbutton1 = jsbutton1;
+	  ojsbutton23 = jsbutton23;
 	}
 #endif
       
@@ -1906,7 +2254,7 @@ void play()
 	      /* Up to throw: */
 	      
 	      eraseyou(playerx);
-	      throw(playerx);
+	      _throw(playerx);
 	    }
 	  else if (key == SDLK_DOWN ||
 		   key == SDLK_z)
@@ -1933,12 +2281,15 @@ void play()
 	      
 	      toggle = 0;
 
-#ifndef NOSOUND	      
-	      for (i = 0; i < MIX_CHANNELS; i++)
-		Mix_HaltChannel(i);
+#ifndef NOSOUND
+	      if (use_sound)
+	      {
+	        for (i = 0; i < MIX_CHANNELS; i++)
+		  Mix_HaltChannel(i);
 	      
-	      if (no_music == 0)
-		Mix_HaltMusic();
+	        if (no_music == 0)
+		  Mix_HaltMusic();
+	      }
 #endif
 
 	      do
@@ -1975,8 +2326,7 @@ void play()
 	      dest.w = 48;
 	      dest.h = 16;
 
-	      SDL_FillRect(window, &dest,
-			   SDL_MapRGB(window->format, 0x00, 0x00, 0x00));
+	      my_FillRect(window, &dest, 0x00, 0x00, 0x00);
 	      SDL_UpdateRect(window, dest.x, dest.y, dest.w, dest.h);
 	    }
 	  
@@ -2019,7 +2369,7 @@ void play()
 	  
 	  if (clicks >= 370 - q && firstround == YES)
 	    {
-	      warning = YES;
+	      _warning = YES;
 	    }
 	  
 	  
@@ -2036,7 +2386,7 @@ void play()
 	      clicks = 0;
 	      firstround = YES;
 	      
-	      warning = NO;
+	      _warning = NO;
 	    }
 	}
       else
@@ -2096,20 +2446,26 @@ void play()
       
       /* Play music: */
 #ifndef NOSOUND
-      if (no_music == 0)
-	{
-	  if (!Mix_PlayingMusic())
-	    Mix_PlayMusic(songs[MOD_GAME1 + (rand() % NUM_GAME_MODS)], 0);
-	  /* Mix_VolumeMusic(4); */
-	}
+      if (use_sound)
+      {
+        if (no_music == 0)
+	  {
+	    if (!Mix_PlayingMusic())
+	      Mix_PlayMusic(songs[MOD_GAME1 + (rand() % NUM_GAME_MODS)], 0);
+	    /* Mix_VolumeMusic(4); */
+	  }
+      }
 #endif
     }
   while (gameover == NO);
   
   
 #ifndef NOSOUND
-  if (no_music == 0)
-    Mix_HaltMusic();
+  if (use_sound)
+  {
+    if (no_music == 0)
+      Mix_HaltMusic();
+  }
 #endif
   
   if (win == NO)
@@ -2162,18 +2518,12 @@ void play()
   do
     {
       getkey(0);
-#ifndef NOJOYSTICK
-      if (js_fd >= 0)
-	{
-	  read(js_fd, &js, JS_RETURN);
-	  if (js.buttons != 0)
-	    key = SDLK_PERIOD;
-	}
-#endif
     }
   while (key == SDLK_LAST);
 
+#ifndef EMBEDDED
   eatjoybuttons();
+#endif
 }
 
 
@@ -2181,7 +2531,12 @@ void play()
 
 void quitapp()
 {
-  /* XCloseDisplay(display); */
+#ifndef EMBEDDED
+  if (use_joystick)
+    SDL_JoystickClose(js);
+  
+  SDL_Quit();
+#endif
 }
 
 
@@ -2201,7 +2556,7 @@ int main(int argc, char * argv[])
   int quit;
   char datafile[1024];
   FILE * fi;
-#ifndef NOSOUND
+#ifndef EMBEDDED
   SDL_version sdlver;
 #endif
   
@@ -2214,26 +2569,23 @@ int main(int argc, char * argv[])
       
       if (strcmp(argv[1], "--version") == 0)
 	{
-	  printf("\nGem Drop X 0.7\n\n");
-	  printf("December 28, 1999\n");
+	  printf("\nGem Drop X 0.9\n\n");
+	  printf("February 11, 2002\n");
 	  printf("by Bill Kendrick\n");
-	  printf("New Breed Software, (c) 1997-1999\n\n");
+	  printf("New Breed Software, (c) 1997-2002\n\n");
 	  
 	  printf("Most graphics by Bernhard Trummer\n");
 	  printf("bernhard.trummer@gmx.net\n\n");
 	  
 	  printf("Visit our website: http://www.newbreedsoftware.com/\n\n");
 	  
-#ifndef NOJOYSTICK
-	  printf("Joystick driver version %d.%d.%d.\n\n",
-		 (JS_VERSION & 0xFF0000) >> 16,
-		 (JS_VERSION & 0x00FF00) >> 8,
-		 (JS_VERSION & 0x0000FF));
-#endif
-
+#ifndef EMBEDDED
 	  SDL_VERSION(&sdlver);
 	  printf("SDL version %d.%d.%d\n\n", sdlver.major,
 		 sdlver.minor, sdlver.patch);
+#else
+	  printf("Built for Qt/Embedded\n\n");
+#endif
 
 	  exit(0);
 	}
@@ -2268,12 +2620,10 @@ int main(int argc, char * argv[])
 	  printf("  Left-click Above Man    - Grab\n");
 	  printf("  Right-click Above man   - Throw\n\n");
 
-#ifndef NOJOYSTICK
 	  printf("Joystick:\n");
 	  printf("  Left/Right - Move left/right\n");
 	  printf("  Button 1/2 - Throw/Grab\n");
 	  printf("  Button 3/4 - Get more gems immediately\n\n");
-#endif
 	  
 	  exit(0);
 	}
@@ -2300,7 +2650,7 @@ int main(int argc, char * argv[])
   
   /* Setup: */
   
-  setup();
+  setup(argc, argv);
   
   
   /* Get max level: */
@@ -2323,7 +2673,7 @@ int main(int argc, char * argv[])
   else
     max_level = 10;
   
-  
+
   /* Main loop: */
   
   do
@@ -2332,8 +2682,11 @@ int main(int argc, char * argv[])
       quit = title();
       
 #ifndef NOSOUND
-  if (no_music == 0)
-    Mix_HaltMusic();
+      if (use_sound)
+	{
+	  if (no_music == 0)
+	    Mix_HaltMusic();
+	}
 #endif
 
       if (quit == NO)
